-- ========================================================================== --
--                      Boss Mod Nameplates                                   --
-- @Author   : Skamer <https://mods.curse.com/members/DevSkamer>              --
-- @Website  : https://wow.curseforge.com/projects/boss-mod-nameplates        --
-- ========================================================================== --
Scorpio               "BossModNameplates.Nameplate"                      "1.2.0"
-- ========================================================================== --
namespace "BMN"

struct "Aura"
  spellID = Number
  desaturated = Boolean
  startTime = Number
  endTime = Number
  duration = Number
  unit = String
  guid = String
  icon = String + Number
  borderColor = Table
endstruct "Aura"

class "Nameplate"
  _NameplateCache = setmetatable( {}, { __mode = "k" } )
-- ======================================================================== --
-- Methods
-- ======================================================================== --
  __Arguments__{}
  function Show(self)
    self.iconFrame:Show()
  end

  __Arguments__{}
  function Hide(self)
    self.iconFrame:Hide()
  end

  __Arguments__{}
  function ShowBorder(self)
    self.iconFrame.icon:SetHeight(60)
    self.iconFrame.icon:SetWidth(60)
  end

  __Arguments__ {}
  function HideBorder(self)
    self.iconFrame.icon:SetHeight(64)
    self.iconFrame.icon:SetWidth(64)
  end

  function SetBorderColor(self, r, g, b, a)
    self.iconFrame:SetBackdropBorderColor(r, g, b, a or 1)
  end

  __Arguments__{ Boolean }
  function SetIconDesaturated(self, desaturated)
    self.iconFrame.icon:SetDesaturated(desaturated)
  end

  __Arguments__ { Number }
  function SetCooldown(self, cd)
    This.SetCooldown(self, GetTime(), cd)
  end

  __Arguments__{ Number, Number }
  function SetCooldown(self, start, cd)
    self.iconFrame.cooldown:SetCooldown(start, cd)
  end

  __Arguments__{}
  function HideCooldown(self)
    self.iconFrame.cooldown:Hide()
  end

  __Arguments__{}
  function ShowCooldown(self)
    self.iconFrame.cooldown:Show()
  end

  __Arguments__{}
  function ShowCooldownNumbers(self)
    self.iconFrame.cooldown:SetHideCountdownNumbers(false)
  end

  __Arguments__{}
  function HideCooldownNumbers(self)
    self.iconFrame.cooldown:SetHideCountdownNumbers(true)
  end

  __Arguments__{ Boolean }
  function SetCooldownReverse(self, reverse)
    self.iconFrame.cooldown:SetReverse(reverse)
  end


  __Arguments__{ Aura }
  function UpdateFromAura(self, aura)
    if aura.startTime == 0 then
      aura.startTime = GetTime()
    end

    if aura.duration == 0 and aura.spellID then
      local guid = aura.guid
      if not guid and aura.unit then
        guid = UnitGUID(aura.unit)
      end

      aura.duration = _Addon:GetAuraDuration(guid, aura.spellID)
      self:SetCooldown(aura.startTime, aura.duration)
    elseif aura.duration > 0 then
      self:SetCooldown(aura.startTime, aura.duration)
    end

    self.icon = aura.icon
    self:SetIconDesaturated(aura.desaturated)
    --self:SetCooldown(aura.startTime, aura.duration)

    if aura.borderColor then
      self:SetBorderColor(aura.borderColor.r, aura.borderColor.g, aura.borderColor.b)
    end
  end

  function Refresh(self)
    self.iconFrame:ClearAllPoints()

    self.iconFrame:SetWidth(Nameplate.defaultWidth)
    self.iconFrame:SetHeight(Nameplate.defaultHeight)
    self.iconFrame:SetPoint(Nameplate.defaultAnchorFrom, self.f, Nameplate.defaultAnchorTo , Nameplate.defaultOffsetX, Nameplate.defaultOffsetY)


    self.iconFrame.icon:SetWidth(Nameplate.defaultWidth - 4)
    self.iconFrame.icon:SetHeight(Nameplate.defaultHeight - 4)
  end

  __Static__() function RefreshAll()
    for obj in pairs(_NameplateCache) do
      obj:Refresh()
    end
  end
  -- ======================================================================== --
  -- Handlers
  -- ======================================================================== --
  local function SetIcon(self, new, old, prop)
    self.iconFrame.icon:SetTexture(new)
  end

  local function SetText(self, new, old, prop)
    self.iconFrame.text:SetText(new)
  end
  -- ======================================================================== --
  -- Properties
  -- ======================================================================== --
  property "icon" { TYPE = String + Number, HANDLER = SetIcon }
  property "text" { TYPE = String, HANDLER = SetText }
  -- DB properties
  __Static__() property "defaultWidth" { HANDLER = RefreshAll }
  __Static__() property "defaultHeight" { HANDLER = RefreshAll }
  __Static__() property "defaultOffsetY" { HANDLER = RefreshAll }
  __Static__() property "defaultOffsetX" { HANDLER = RefreshAll }
  __Static__() property "defaultAnchorFrom" { HANDLER = RefreshAll }
  __Static__() property "defaultAnchorTo" { HANDLER = RefreshAll }
  -- ======================================================================== --
  -- Constructors
  -- ======================================================================== --
  function Nameplate(self, f)
    local iconFrame = CreateFrame("Frame", nil, UIParent)
    iconFrame:SetHeight(Nameplate.defaultHeight)
    iconFrame:SetWidth(Nameplate.defaultWidth)
    iconFrame:SetFrameStrata("WORLD")
    iconFrame:SetPoint(Nameplate.defaultAnchorFrom, f, Nameplate.defaultAnchorTo, Nameplate.offsetX, Nameplate.offsetY)
    iconFrame:SetBackdrop({
      edgeFile = [[Interface\AddOns\BossModNameplates\Media\Textures\Frame-Border]],
      tile = false, tileSize = 32, edgeSize = 8,
      insets = { left = 0, right = 0, top = 0, bottom = 0 }
    })
    iconFrame:SetBackdropBorderColor(1, 0, 0, 1)

    local icon = iconFrame:CreateTexture()
    icon:SetHeight(Nameplate.defaultHeight - 4)
    icon:SetWidth(Nameplate.defaultWidth - 4)
    icon:SetPoint("CENTER")
    icon:SetTexture(GetSpellTexture(205429))
    icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
    iconFrame.icon = icon

    local text = iconFrame:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
    text:SetTextColor(1, 0, 0)
    text:SetPoint("CENTER")
    text:SetText("")
    text:Hide()
    iconFrame.text = text

    local cooldown = CreateFrame("Cooldown", nil, iconFrame, "CooldownFrameTemplate")
    cooldown:SetAllPoints(icon)
    iconFrame.cooldown = cooldown

    f.dbtex = f:CreateTexture()
    f.dbtex:SetAllPoints()
    f.dbtex:SetColorTexture(1, 0, 0)
    f.dbtex:Hide()

    f.BMN = self
    self.f = f
    self.iconFrame = iconFrame

    self:SetIconDesaturated(true)
    self:SetCooldownReverse(true)
    self:Hide()

    _NameplateCache[self] = true
  end

endclass "Nameplate"

-- ========================================================================== --
function OnLoad(self)

end

function OnDisable(self)

end

function OnEnable(self)

end
